/*
   This is one good way to handle OpenGL extensions and OpenGL 4 features.

   It is based on an idea of Jerry Dekker and it uses the glext.h header
   in addition to this header and the header Dekker2.h.

   The other good way to handle OpenGL extensions is to use glew.h. The
   main advantage to glew.h is that everything done below is done for you
   automatically. The main disadvantage to using glew.h is that everything
   done below is done for you automatically. When you do it yourself, you
   are more aware of what is going on, and you have a better chance of
   figuring out what is wrong when it all blows up in your face.

   To use this technique, you first need to declare, in the first block of
   code below, the name of any OpenGL extension function that you plan to
   use. You need to declare the function using its type as defined in the
   file glext.h (so you need to look up your function in that header file
   and find its type).

   Second, you need to add a line for your function into the second block
   of code below. The line should be of the form
      GB_GL(FUNCTION_TYPE, function_name);
   where, again, the function type comes from glext.h.

   Third, include this header file into your OpenGL file that contains
   your main() function, and then, after your code that initializes GLUT
   (or whatever is creating your OpenGL graphics context), call the
   function populate_functions() using the following few lines of code.

   if ( !populate_functions() )
   {
      fprintf(stderr, "Some OpenGL extensions not supported.\n");
      exit(0);
   }

   Fourth, if you have any other compilation units, include the header
   Dekker2.h into each of them.

   For the latest version of glext.h, see
      http://www.opengl.org/registry/#headers
*/
#ifndef DEKKER1_H
#define DEKKER1_H

#include "glext.h"

#ifndef DEKKER2_H
#define STORAGE
#endif

/* Declarations of OpenGL function names using types defined in glext.h. */
STORAGE PFNGLGENBUFFERSPROC              glGenBuffers;
STORAGE PFNGLBINDBUFFERPROC              glBindBuffer;
STORAGE PFNGLBUFFERDATAPROC              glBufferData;
STORAGE PFNGLBUFFERSUBDATAPROC           glBufferSubData;
STORAGE PFNGLDELETEBUFFERSPROC           glDeleteBuffers;
STORAGE PFNGLGETBUFFERPARAMETERIVPROC    glGetBufferParameteriv;
STORAGE PFNGLMAPBUFFERPROC               glMapBuffer;
STORAGE PFNGLUNMAPBUFFERPROC             glUnmapBuffer;
STORAGE PFNGLDRAWRANGEELEMENTSPROC       glDrawRangeElements;
STORAGE PFNGLSHADERSOURCEPROC            glShaderSource;
STORAGE PFNGLCOMPILESHADERPROC           glCompileShader;
STORAGE PFNGLCREATESHADERPROC            glCreateShader;
STORAGE PFNGLGETSHADERIVPROC             glGetShaderiv;
STORAGE PFNGLGETSHADERINFOLOGPROC        glGetShaderInfoLog;
STORAGE PFNGLCREATEPROGRAMPROC           glCreateProgram;
STORAGE PFNGLATTACHSHADERPROC            glAttachShader;
STORAGE PFNGLLINKPROGRAMPROC             glLinkProgram;
STORAGE PFNGLGETPROGRAMIVPROC            glGetProgramiv;
STORAGE PFNGLGETPROGRAMINFOLOGPROC       glGetProgramInfoLog;
STORAGE PFNGLGETUNIFORMLOCATIONPROC      glGetUniformLocation;
STORAGE PFNGLUSEPROGRAMPROC              glUseProgram;
STORAGE PFNGLUNIFORM2FPROC               glUniform2f;
STORAGE PFNGLUNIFORM3FPROC               glUniform3f;


/*
   At runtime(!), assign values to the function names.

   The Win32 function wglGetProcAddress() is what
   actually looks up the appropriate value for each
   function name.

   http://msdn.microsoft.com/en-us/library/dd374386(v=vs.85).aspx
*/
#ifndef DEKKER2_H

  unsigned populate_functions(void)
  {
     unsigned ok = 1;

     // Look up the function name n with type t.
     #define GB_GL(t,n)\
        ok &= (n = (t)wglGetProcAddress(#n)) ? 1 : printf("no "#n" found.\n") & 0;

     GB_GL(PFNGLGENBUFFERSPROC,glGenBuffers);
     GB_GL(PFNGLBINDBUFFERPROC,glBindBuffer);
     GB_GL(PFNGLBUFFERDATAPROC,glBufferData);
     GB_GL(PFNGLBUFFERSUBDATAPROC,glBufferSubData);
     GB_GL(PFNGLDELETEBUFFERSPROC,glDeleteBuffers);
     GB_GL(PFNGLGETBUFFERPARAMETERIVPROC,glGetBufferParameteriv);
     GB_GL(PFNGLMAPBUFFERPROC,glMapBuffer);
     GB_GL(PFNGLUNMAPBUFFERPROC,glUnmapBuffer);
     GB_GL(PFNGLDRAWRANGEELEMENTSPROC,glDrawRangeElements);
     GB_GL(PFNGLSHADERSOURCEPROC,glShaderSource);
     GB_GL(PFNGLCOMPILESHADERPROC,glCompileShader);
     GB_GL(PFNGLCREATESHADERPROC,glCreateShader);
     GB_GL(PFNGLGETSHADERIVPROC,glGetShaderiv);
     GB_GL(PFNGLGETSHADERINFOLOGPROC,glGetShaderInfoLog);
     GB_GL(PFNGLCREATEPROGRAMPROC,glCreateProgram);
     GB_GL(PFNGLATTACHSHADERPROC,glAttachShader);
     GB_GL(PFNGLLINKPROGRAMPROC,glLinkProgram);
     GB_GL(PFNGLGETPROGRAMIVPROC,glGetProgramiv);
     GB_GL(PFNGLGETUNIFORMLOCATIONPROC,glGetUniformLocation);
     GB_GL(PFNGLUSEPROGRAMPROC,glUseProgram);
     GB_GL(PFNGLUNIFORM2FPROC,glUniform2f);
     GB_GL(PFNGLUNIFORM3FPROC,glUniform3f);
     return ok;
  }

#endif /* not defined DEKKER2_H */

#endif /* DEKKER1_H */
