#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "glut.h"
#include "Dekker2.h"


// Used to check for fatal programming errors.
void checkOpenGLError(int line)
{
   int wasError = 0;
   GLenum error = glGetError();
   while (error != GL_NO_ERROR)
   {
      printf("GL ERROR: at line %d: %s\n", line, gluErrorString(error));
      wasError = 1;
      error = glGetError();
   }
   if (wasError) exit(-1);
}

//================ Shader code ================================

// GLSL source code for our vertex shader.
const GLchar *vertexShaderSource =
{
  "uniform vec2 glyphCenter;\n"
  "uniform vec2 glyphSize;\n"
  "void main() {\n"
  "  float aspect = glyphSize.x / glyphSize.y;\n"
  "  vec2 scale = vec2(2.0*aspect/glyphSize.x, 2.0/glyphSize.y);\n"
  "  gl_Position = vec4(scale*(gl_Vertex.xy - glyphCenter), 0.0, 1.0);\n"
  "}\n"
};


// GLSL source code for our fragment shader.
const GLchar *fragmentShaderSource =
{
  "uniform vec3 glyphColor;\n"
  "void main() {\n"
  "  gl_FragColor = vec4(glyphColor, 1.0);\n"
  "}\n"
};


// Handles for shaders and program.
GLuint vertexShader;
GLuint fragmentShader;
GLuint program;


// Handles for uniform variables.
GLint glyphCenter;
GLint glyphSize;
GLint glyphColor;


// Install our shader programs and tell GL to use them.
// We also initialize the uniform variables.
void installShaders(void)
{
   // (1) Create shader objects
   vertexShader = glCreateShader(GL_VERTEX_SHADER);
   fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);


   // (2) Load source code into shader objects.
   glShaderSource(vertexShader, 1, &vertexShaderSource, NULL);
   glShaderSource(fragmentShader, 1, &fragmentShaderSource, NULL);


   // (3) Compile shaders.
   glCompileShader(vertexShader);
   GLint infoLogLength;
   glGetShaderiv(vertexShader, GL_COMPILE_STATUS, &infoLogLength);
   if (infoLogLength > 1)
   {
      GLchar infoLog[infoLogLength+1];
      GLint charsWritten;
      glGetShaderInfoLog(vertexShader, infoLogLength, &charsWritten, infoLog);
      fprintf(stderr, "shader info log:\n%s\n\n", infoLog);
   }
   checkOpenGLError(__LINE__);

   glCompileShader(fragmentShader);
   glGetShaderiv(fragmentShader, GL_COMPILE_STATUS, &infoLogLength);
   if (infoLogLength > 1)
   {
      GLchar infoLog[infoLogLength+1];
      GLint charsWritten;
      glGetShaderInfoLog(fragmentShader, infoLogLength, &charsWritten, infoLog);
      fprintf(stderr, "shader info log:\n%s\n\n", infoLog);
   }
   checkOpenGLError(__LINE__);


   // (4) Create program object and attach vertex and fragment shader.
   program = glCreateProgram();
   glAttachShader(program, vertexShader);
   glAttachShader(program, fragmentShader);
   checkOpenGLError(__LINE__);


   // (5) Bind vertex attributes, varying variables, and fragment shader outputs.
   // We are using the "built-in" values gl_Vertex, gl_Position, and
   // gl_Fragment so there is nothing to do here.


   // (6) Link program.
   glLinkProgram(program);
   glGetProgramiv(program, GL_LINK_STATUS, &infoLogLength);
   if (infoLogLength > 1)
   {
      GLchar infoLog[infoLogLength+1];
      GLint charsWritten;
      glGetProgramInfoLog(program, infoLogLength, &charsWritten, infoLog);
      fprintf(stderr, "program info log:\n%s\n\n", infoLog);
   }
   checkOpenGLError(__LINE__);


   // (7) Fetch handles for uniform variables in program.
   glyphCenter = glGetUniformLocation(program, "glyphCenter");
   glyphSize = glGetUniformLocation(program, "glyphSize");
   glyphColor = glGetUniformLocation(program, "glyphColor");


   // (8) Tell GL to use our program
   glUseProgram(program);


   // (9) Set the uniform variables used by our program.
   glUniform2f(glyphCenter, 1.5, 2.5);
   glUniform2f(glyphSize, 3.0, 5.0);
   glUniform3f(glyphColor, 1.0, 0.0, 0.0);
}
